//********************************************************
// Class Button
// Display a button
//
// str value: Value to display in the button
// dom parent: in which dom element to draw the button
// int id: id of the dom element
// func onclick: callback function when button clicked
// str className : className of the general button
// str classNameText : className for the button's text 
//********************************************************

var Button = Class.create( {
    initialize: function( param )
    {
        // Init
        this.id = param.id ? param.id : "";
        this.parent = param.parent ? param.parent : false;
        this.value = param.value ? param.value : "";
        this.className = param.className ? param.className : false;
        this.classNameText = param.classNameText ? param.classNameText : false;
        this.callbackOnClick = param.onClick ? param.onClick : false;
        this.callbackOnMouseOver = param.onMouseOver ? param.onMouseOver : false;
        this.callbackOnMouseOut = param.onMouseOut ? param.onMouseOut : false;
        this.boolDisabled = false;
        this.boolSelected = false;
        this.boolDisplayed = false;

        // Create button elements
        this.divContainer = $( document.createElement( "button" ) );
        if( this.className )
            this.divContainer.className = this.className;
        this.divContainer.atMe = this;
        this.divContainer.id = this.id;
        if( this.parent )
            this.parent.appendChild( this.divContainer );

        this.divText = $( document.createElement( "div" ) );
        if( this.classNameText )
            this.divText.className = "button_text";
        this.divText.innerHTML = this.value;
        this.divContainer.appendChild( this.divText );

        // Define button events
        Event.observe( this.divContainer, 'click', this.onClick.bindAsEventListener( this ) );
        Event.observe( this.divContainer, 'mouseover', this.onHover.bindAsEventListener( this ) );
        Event.observe( this.divContainer, 'mouseout', this.onOut.bindAsEventListener( this ) );
    },

    // Getter / Setter ********************************************************

    getContainer: function()
    {
        return this.divContainer;
    },

    isDisable : function()
    {
        return this.boolDisabled;
    },

    isSelected: function()
    {
        return this.boolSelected;
    },

    setValue : function( value )
    {
        this.value = value;
        this.divText.innerHTML = this.value;
    },

    setCallbackOnClick : function( value )
    {
        this.callbackOnClick = value;
    },

    // Actions ********************************************************

    disable : function()
    {
        this.boolDisabled = true;
        this.divContainer.addClassName( "disable" );
    },

    enable : function()
    {
        this.boolDisabled = false;
        this.divContainer.removeClassName( "disable" );
    },

    select : function( value )
    {
        this.boolSelected = value;
        if( this.boolSelected )
            this.divContainer.addClassName( "selected" );
        else
            this.divContainer.removeClassName( "selected" );
    },

    show : function()
    {
        this.boolDisplayed = true;
        this.divContainer.style.display = "";
    },

    hide : function()
    {
        this.boolDisplayed = false;
        this.divContainer.style.display = "none";
    },

    // Events ********************************************************

    onClick : function( event )
    {
        if( !this.isDisable() && (undefined == event.detail || 1 == event.detail) )
        {
            if( this.callbackOnClick )
                this.callbackOnClick();
        }
        Event.stop( event );
    },

    onHover : function( event )
    {
        if( !this.isDisable() && this.callbackOnMouseOver )
            this.callbackOnMouseOver();

        Event.stop( event );
    },

    onOut: function()
    {
        if( !this.isDisable() && this.callbackOnMouseOut )
            this.callbackOnMouseOut();
    }
} );