package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.Mandatory;
import com.ether.annotation.ParamName;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import com.medias.database.objects.TypePlateforme;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 17 feb 2011
 */
public class Controller
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /**
     * ********************************************************** *
     */
    // Default view if methodName is unknown
    public ModelAndView home( final HttpServletRequest request, final HttpServletResponse response )
            throws EtherException
    {
        return new ModelAndView( "index" );
    }

    @ControllerMethod(view = VIEW_VISUALIZATION)
    public Map<String, Object> view()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM)
    public Map<String, Object> viewParametersByPlateform()
            throws ServiceException
    {
        final List<Plateforme> plateforms = _etherService.getAllPlateforms();
//        final List<Plateforme> plateforms = new ArrayList<Plateforme>();
//        final TypePlateforme type = new TypePlateforme( 1, "FIXE" );
//        final Plateforme pf = new Plateforme();
//        pf.setPlateformeId( 1 );
//        pf.setPlateformeNom( "LHVP" );
//        pf.setTypePlateforme( type );
//        plateforms.add( pf );
//        final Plateforme pf2 = new Plateforme();
//        pf2.setPlateformeId( 2 );
//        pf2.setPlateformeNom( "SIRTA" );
//        pf2.setTypePlateforme( type );
//        plateforms.add( pf2 );

        final Map<String, Object> model = new HashMap<String, Object>();
        model.put( "plateforms", getJsonHelper().toJSON( plateforms ) );
        model.put( "axeTypesForFixedPlateforms", getJSONAxeTypesForFixedPlateforms() );
        model.put( "axeTypesForMobilePlateforms", getJSONAxeTypesForMobilePlateforms() );
        return model;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /**
     * ********************************************************** *
     */
    @ControllerMethod(jsonResult = true)
    public JSONObject searchParametersByPlateform( @Mandatory @ParamName(ParameterConstants.PARAMETER_ID) final Integer plateformId )
            throws ServiceException, EtherException
    {
        final List<Parametre> parametersByPlateform = _etherService.getParametersByPlateformId( plateformId );
//        final List<Parametre> parametersByPlateform = new ArrayList<Parametre>( 2 );
//        final Parametre p1 = new Parametre();
//        p1.setParametreId( 1 );
//        p1.setParametreNom( "JNO2" );
//        parametersByPlateform.add( p1 );
//        final Parametre p2 = new Parametre();
//        p2.setParametreId( 2 );
//        p2.setParametreNom( "CO2" );
//        parametersByPlateform.add( p2 );

        final JSONObject result = new JSONObject();
        result.put( ParameterConstants.PARAMETER_PARAMETERS, getJsonHelper().toJSON( parametersByPlateform ) );
        return result;
    }

    private List<JSONObject> getJSONAxeTypesForFixedPlateforms()
    {
        final AxeTypeForFixedPlateform[] axeTypes = AxeTypeForFixedPlateform.values();

        final List<JSONObject> jsonAxeTypes = new ArrayList<JSONObject>( axeTypes.length );

        for( final AxeTypeForFixedPlateform axeType : axeTypes )
        {
            final JSONObject jsonAxeType = new JSONObject();
            jsonAxeType.put( "text", axeType.name() );
            jsonAxeType.put( "value", axeType.name() );
            jsonAxeTypes.add( jsonAxeType );
        }
        return jsonAxeTypes;
    }

    private List<JSONObject> getJSONAxeTypesForMobilePlateforms()
    {
        final AxeTypeForMobilePlateform[] axeTypes = AxeTypeForMobilePlateform.values();

        final List<JSONObject> jsonAxeTypes = new ArrayList<JSONObject>( axeTypes.length );

        for( final AxeTypeForMobilePlateform axeType : axeTypes )
        {
            final JSONObject jsonAxeType = new JSONObject();
            jsonAxeType.put( "text", axeType.name() );
            jsonAxeType.put( "value", axeType.name() );
            jsonAxeTypes.add( jsonAxeType );
        }
        return jsonAxeTypes;
    }

    @Required
    public void setEtherService( @NotNull final EtherService etherService )
    {
        _etherService = etherService;
    }

    private static final Log LOGGER = LogFactory.getLog( Controller.class );

    private static final String VIEW_VISUALIZATION = "visualization/visu";
    private static final String VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM = "visualization/visu_parameter_by_pf";

    private EtherService _etherService;
}